#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Script para descargar archivos desde goear.
# Copyright (C) 2007-2010 Daniel Fuentes B.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ===================================================================
# Importamos algunos módulos que luego usaremos en el script:
# ===================================================================

import urllib
from xml.dom import minidom
import sys
import os.path
from os import curdir

# ===================================================================
# Algunas variables a usar:
# ===================================================================

__version__ = "0.8"
quiet_mode = False

# El texto que se muestra en la ayuda
texto_ayuda = """Usage: goear-dl.py [Opción] [URL or ID]

URL Format (or ID)

Please insert a valid URL, similar to:
http://www.goear.com/listen.php?v=xxxxxxx
http://www.goear.com/listen/xxxxxxx/song-author
Or enter the ID of the file (has 7 number/letters)

Options:
-h, --help        print this help text and exit
-v, --version     print program version and exit
-i, --list FILE   download URLs/IDs found in local FILE
                  (one URL/ID per line in FILE)
-q, --quiet       activates quiet mode

Examples of usage:

** Say you want to download a URL. Just type one of these:
goear-dl.py http://www.goear.com/listen.php?v=xxxxxxx
goear-dl.py http://www.goear.com/listen/xxxxx/song-author
goear-dl.py xxxxxxx

** Download a URL/ID in quiet mode:
goear-dl.py -q http://www.goear.com/listen.php?v=xxxxx
goear-dl.py -q http://www.goear.com/listen/xxxxx/song-author

** Read url's from a local file and retrieve them:
goear-dl.py --list file_with_urls.txt

For more info check "README" file."""


# ===================================================================
# Funciones y clases usadas en el programa:
# ===================================================================


class MyOpener(urllib.FancyURLopener):
    """Clase que cambia el user agent que usa python"""
    # Por defecto python usa: URLopener.version = "Python-urllib/1.xx"
    # lo cambiamos por el de IE 7
    version = "Mozilla/5.0 (Windows; U; MSIE 7.0; Windows NT 6.0; en-US)"
    # Ver http://docs.python.org/library/urllib.html#urllib._urlopener


class Goear():
    """Clase que obtiene la informacion del archivo (nombre, artista, etc)"""

    def __init__(self, audio_id):
        self.id = audio_id
        self.data = None
        # Para que funcione la clase, al incio hay que obtener la informacion
        # del archivo xml a partir de la ID (primero abrir el xml)
        url_xml = "http://www.goear.com/tracker758.php?f=" + self.id

        # Cambiamos el user agent por el definido anteriormente:
        urlopen = MyOpener().open
        try:
            xmldoc = urlopen(url_xml, None)
            xml_data = minidom.parse(xmldoc)
            xmldoc.close()
            data = xml_data.toxml()
        # Mensaje en caso de error, o de no encontrarse la ruta
        except:
            print "[goear] Error: Failed to obtain information from Goear"
            sys.exit(1)

        self.data = data  # Contiene la copia de informacion del xml

    def get_id(self):
        "muestra la ID de la cancion"
        return self.id

    def url_file(self):
        "devuelve la url del archivo mp3"
        # Buscar la linea "song path=" que es la dirección del archivo
        inicio_path = self.data.find("path=")
        fin_path = self.data.find(".mp3")
        path = self.data[(inicio_path + 6):(fin_path + 4)]
        return path

    def artist(self):
        "devuelve el grupo/artista de esta cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("artist=")
        fin = self.data.find("\"", (inicio + 8), -1)
        artist = self.data[(inicio + 8):fin]
        return artist

    def title(self):
        "devuelve el titulo de la cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("title=")
        fin = self.data.find("\"", (inicio + 7), -1)
        title = self.data[(inicio + 7):fin]
        return title

# -------------------------------------------------------------------
# Definimos la función de descarga:


def download_file(filename, URL, quiet=False):
    # Cambiamos el user agent por el definido anteriormente:
    urlretrieve = MyOpener().retrieve
    if quiet == True:
        f = urlretrieve(URL, filename)
    else:
        f = urlretrieve(URL, filename, reporthook=my_report_hook)


def my_report_hook(block_count, block_size, total_size):
    """función que muestra el progreso del archivo"""
    # Usada si al descargar se pide mostrar el progreso (por defecto)
    total_kb = total_size / 1024
    descargado = block_count * (block_size / 1024)
    porcentaje = descargado * 100 / total_kb
    # Mostramos en el teminal la salida, se usa stdout porque no
    # inserte el alto de linea "\n" a diferencia de print
    texto_barra = "[download] " + str(descargado) + " kb of " +\
    str(total_kb) + " kb (" + str(porcentaje) + "%)"
    # Borramos la linea anterior (asi se actualiza la misma linea)
    sys.stdout.write("\b" * (len(texto_barra)))
    sys.stdout.write(texto_barra)
    sys.stdout.flush()


# -------------------------------------------------------------------
# Funcion que obtiene la ID a partir de la dirección ingresada


def get_id(audio_url):
    # hay varios posibles casos:
    # a) Se ingresa directamente la ID (codigo de 7 numeros/letras)
    # b) De la forma antigua (http://www.goear.com/listen.php?v=xxxxx):
    # c) De la forma nueva (http://www.goear.com/listen/xxxxx/cancion-artista)

    if len(audio_url) == 7:
        # se ingreso la id directamente (primer caso)
        audio_id = audio_url
    elif (audio_url.find("listen.php?v=")) != -1:  # Segundo caso
        # de la forma antigua el find deberia encontrar el "listen.php?v="
        # siendo distintinto de -1 (es -1 cuando no encuentra el texto pedido)
        inicio_id = audio_url.find("v=")
        audio_id = audio_url[(inicio_id + 2):(inicio_id + 9)]
    elif (audio_url.find("listen/")) != -1:
        # la nueva forma (tercer caso)
        inicio_id = audio_url.find("listen/")
        audio_id = audio_url[(inicio_id + 7):(inicio_id + 14)]
    else:
        print "[goear-dl] Error: Not find the address or ID for:\n", audio_url
        audio_id = None

    return audio_id


# -------------------------------------------------------------------
# Funcion que lee un archivo de texto y devuelve su contenido


def filelist(filename):
    # Se busca el filename en el directorio actual
    if os.path.exists(os.path.join(os.curdir, filename)):
        lista_lineas = []
        fichero = open(filename, "r")
        for line in fichero:
            lista_lineas.append(line.rstrip())  # rstrip() Elimina \n al final
        fichero.close()
        return lista_lineas
    # Se supone que el filename es la ruta completa de la lista
    elif os.path.exists(filename):
        lista_lineas = []
        fichero = open(filename, "r")
        for line in fichero:
            lista_lineas.append(line.rstrip())
        fichero.close()
        return lista_lineas
    else:  # No se encuentra la ruta del archivo
        print "[goear-dl] Error: %s not found." % (filename)
        sys.exit(1)


# -------------------------------------------------------------------
# funcion que comprueba las opciones ingresadas, actua de acuerdo a ella y
# regresa la url de goear (si es necesaria) o termina el programa


def checkoptions():
    """Comprueba las opciones, las ejecuta y regresa las audio_url"""
    # Variables a usar en la funcion (una ya esta definida, la global)
    global quiet_mode
    opciones_ingresadas = []
    audio_url_list = []

# -----------------------------------------------
# Primero se comprueba la entrada de datos, hay dos posibles casos:
# a) Solo una opcion o la URL/ID, b) muchas opciones

    # Caso 1: Se ha ingresa solo 1 opcion (o la url)
    if len(sys.argv) == 2:
    # Para el caso 1, hay dos valores posibles:
    # se ingreso una opcion o se ingreso solo la url sin ninguna opcion
        if (sys.argv[1] == "-q" or sys.argv[1] == "--quiet" or
sys.argv[1] == "-h" or sys.argv[1] == "--help" or sys.argv[1] == "help" or
sys.argv[1] == "-v" or sys.argv[1] == "--version" or sys.argv[1] == "moo"):
            opciones_ingresadas.append(sys.argv[1])
        # como no hay url no se agrega nada a audio_url_list
        elif (sys.argv[1] == "--list"):
            print "[goear-dl] Error: Please enter the list filename \
in the option \"--list\"."
            sys.exit(1)
        else:
            audio_url_list.append(sys.argv[1])

    # Caso 2: Se ingresan muchas opciones
    elif len(sys.argv) > 2:
    # se pueden tener solo ordenes o tener opciones (ordenes) + dirección
        ingresados = sys.argv[1:len(sys.argv)]
        m = 0
        for n in ingresados:
            m = m + 1
            if (n == "-q" or n == "--quiet" or n == "-h" or n == "--help"
or n == "help" or n == "-v" or n == "--version" or n == "moo"):
                opciones_ingresadas.append(n)
            elif (n == "--list" or n == "-i"):
                lista_archivos = filelist(ingresados[m])
                for elementos in lista_archivos:
                    audio_url_list.append(elementos)
                # remover el archivo que contiene lista con urls
                del ingresados[m]
            else:
                audio_url_list.append(n)

# -----------------------------------------------
# Luego hay que verificar las Opciones de la linea de comandos
# y proceder de acuerdo a ellas
# (se ignora la opcion --list, ya que se agregaron la urls anteriomente)
    if opciones_ingresadas != []:
        for opcion in opciones_ingresadas:
            if opcion == "-h" or opcion == "--help" or opcion == "help":
                print texto_ayuda
                sys.exit()
            elif opcion == "-v" or opcion == "--version":
                print "[goear-dl] Version:", __version__
                sys.exit()
            elif opcion == "-q" or opcion == "--quiet":
                quiet_mode = True
            elif opcion == "moo":
                import time
                print "No hay ninguna Súper Vaca en este programa, pero"
                print "tenemos una danza:"
                sys.stdout.write(" <(^.^<) ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" <(^-^)> ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" (>^.^)> ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" ^(^.^)^\n")
                sys.stdout.flush()
                sys.exit()

# -----------------------------------------------
# Por ultimo comprobar la dirección (url) del audio sea correcta:
# Puede darse casos como el que sigue: goear.py -q -q (bien raro)
# en donde audio_url_list estaria vacia.
    if audio_url_list == []:
        audio_url_list.append(str(raw_input("[goear-dl] Error: you \
must provide at least one URL or ID:\n")))

# Finalmente la funcion devuelve la lista de las ID (o las URL) de
# cada cancion, pero no comprueba si son validas, despues hay que
# aplicar la funcion get_id() para comprobar si son validas y despues
# se debe aplicar la funcion para obtener infomacion y la de descarga
    return audio_url_list


#====================================================================
# Iniciamos el programa
#====================================================================

if __name__ == "__main__":
    songs_list = []
    # Comprobar a continuación si se ingreso algun dato en la linea de ordenes
    # Si no se ingresa nada sys.argv solo tendra el nombre del script (1 dato)
    if len(sys.argv) == 1:
        # No hay ordenes o url, por lo que hay que pedir la dirección
        songs_list.append(str(raw_input("[goear-dl] Error: you must \
provide at least one URL or ID:\n")))
    else:
        songs_list = checkoptions()  # funcion devuelve las ID

    # Se revisa la lista con las url de goear, para cada una obtiene la
    # ID de cada cancion y luego se procede con la descarga:
    for audio_url in songs_list:
        audio_id = get_id(audio_url)
        if audio_id == None:
            pass  # se pasa a la siguinete en la lista (esta no es valida)
        else:
            cancion = Goear(audio_id)
            if quiet_mode == False:
                print "[goear] Extracting audio information for: %s" % (
                    audio_id)
            # Creamos el nombre del archivo
            nombre_archivo = (cancion.get_id() + " - " + cancion.title() +
                " - " + cancion.artist() + ".mp3")
            if quiet_mode == False:
                print "[download] %s: Destination: \"%s\"" % (audio_id,
                    nombre_archivo)
            # Iniciamos la descarga, download_file() muestra el progreso
            download_file(nombre_archivo, cancion.url_file(), quiet_mode)
            # Mensaje avisando de que se termino la descarga
            print "\n[download] Done. Open the file: \"%s\"" % (
                nombre_archivo)
